package org.example.customization.parser;

import org.apache.calcite.avatica.util.Casing;
import org.apache.calcite.sql.SqlNode;
import org.apache.calcite.sql.SqlWriterConfig;
import org.apache.calcite.sql.dialect.OracleSqlDialect;
import org.apache.calcite.sql.parser.SqlParseException;
import org.apache.calcite.sql.parser.SqlParser;
import org.apache.calcite.sql.pretty.SqlPrettyWriter;
import org.apache.calcite.sql.validate.SqlConformanceEnum;
import org.example.sql.extend.CalciteSqlExtendParserImpl;

public class Main {
    public static void main(String[] args) throws SqlParseException {
        // Sql语句
        String sql = "CREATE catalog test_catalog WITH ('name'='test_catalog','comment'='test')";
        // 解析配置
        SqlParser.Config configBuilder =
                SqlParser.config()
                        .withParserFactory(CalciteSqlExtendParserImpl.FACTORY)
                        .withUnquotedCasing(Casing.UNCHANGED)
                        .withQuotedCasing(Casing.UNCHANGED)
                        .withCaseSensitive(false)
                        .withConformance(SqlConformanceEnum.MYSQL_5);
        // 创建解析器
        SqlParser parser = SqlParser.create(sql, configBuilder);
        // 解析sql
        SqlNode sqlNode = parser.parseQuery();
        SqlWriterConfig SQL_WRITER_CONFIG =
                SqlPrettyWriter.config()
                        .withAlwaysUseParentheses(true)
                        .withUpdateSetListNewline(false)
                        .withFromFolding(SqlWriterConfig.LineFolding.TALL)
                        .withIndentation(0)
                        .withQuoteAllIdentifiers(false)
                        .withKeywordsLowerCase(true)
                        .withDialect(OracleSqlDialect.DEFAULT);
        // 还原某个方言的SQL
        System.out.println(sqlNode.toSqlString(c->SQL_WRITER_CONFIG));
    }
}
